from random import random, randint
import time

# pyglet
#import pyglet
#from pyglet.gl import *
#from pyglet.window.key import *
from pyglet.window import key

# cocos
import cocos
from cocos.director import director
from cocos.sprite import Sprite
from cocos.actions.interval_actions import MoveBy, MoveTo, ScaleBy, JumpBy, RotateBy, Delay, Accelerate, FadeIn, FadeOut
from cocos.actions.instant_actions import Hide, Show, Place, CallFunc
from cocos.actions.base_actions import Reverse, Repeat
from cocos.text import Label
from cocos.layer.util_layers import ColorLayer

import soundex
from constants import *
import level_engine


def log(txt):
    print txt

class BackgroundLayer(cocos.layer.Layer):
    def __init__(self):
        super(BackgroundLayer, self).__init__()
        bg = Sprite('pavement1a.png', position=(WIDTH/2, HEIGHT/2))
        #bg = Sprite('space2a.png', position=(WIDTH/2, HEIGHT/2))
        self.add(bg)

def is_inside(sprite, x_, y_):
    x = sprite.x
    y = sprite.y
    w2 = sprite.width / 2
    h2 = sprite.height / 2
    if x_ > x - w2 and x_ < x + w2 and y_ > y - h2 and y_ < y + h2:
        return True
    return False

class CtrlLayer(cocos.layer.Layer):
    is_event_handler = True
    def __init__(self, scene, level):
        self.scene = scene
        super(CtrlLayer, self).__init__()
        self.level_num = level
        self.level = level_engine.levels[level]()
        self.level.render(self)

        self.keys = set()

        # build layer
        self.add(Label("Level %d" % (self.level_num + 1), (20, HEIGHT-30), bold=True, font_size=20, font_name=FONTS))
        self.add(Label("ESC to exit", (20, 30), bold=True, font_size=16, font_name=FONTS))

        self.time_lbl = Label("Time 00:00", (200, HEIGHT-30), bold=True, font_size=20, font_name=FONTS)
        self.add(self.time_lbl)
        self.time_start = time.time()
        self.time = 0

        self._flash_layer = ColorLayer(255, 255, 255, 0)
        self.scene.add(self._flash_layer, z=10)
        self._is_flashing = False
        self.flash_lbl = Label("", (400, HEIGHT-30), bold=True, font_size=20, font_name=FONTS)
        self.add(self.flash_lbl)

        self.msgs = 0
        self.show_msg("Welcome to the game.\nWatch out!!!\nTime comes!!!\n                             Master")

        self.schedule_interval(self._update, FRAME_PERIOD) # 0.005 second

    def on_mouse_press (self, x, y, buttons, modifiers):
        print x, y
        #if is_inside(self.grain_cnt.sprite, x, y) and self.cash_cnt.cnt >= 10:
            
    def on_key_press(self, k, m):
        if k in [key.UP, key.RIGHT, key.DOWN, key.LEFT]:
            self.keys.add(k)
        elif k == key.ENTER:
            return True
        return False

    def on_key_release(self, k, m):
        if k in [key.UP, key.RIGHT, key.DOWN, key.LEFT]:
            self.keys.remove(k)
        elif k == key.ENTER:
            return True
        return False

    def _update(self, dt):
        time_delta = int(time.time() - self.time_start)
        if time_delta != self.time:
            self.time = time_delta
            self.time_lbl.element.text = "Time %02d:%02d" % (int(self.time / 60), self.time % 60)

        if key.UP in self.keys:
            self.level.move_avatar(0)
        if key.RIGHT in self.keys:
            self.level.move_avatar(1)
        if key.DOWN in self.keys:
            self.level.move_avatar(2)
        if key.LEFT in self.keys:
            self.level.move_avatar(3)

        status = self.level.update(self)

        if status == GAME_NEXT_LEVEL:
            if self.level_num + 1 < len(level_engine.levels):
                next_level = self.level_num + 1
                gs = GameScene(next_level)
                ts = TransitScene("Next level %d" % (next_level+1), font_size=48, width=600, next=gs)
                director.replace(ts)
            else:
                s = TransitScene("Game Completed", font_size=48, width=650, next=1)
                director.replace(s)
        elif status == GAME_OVER:
            s = TransitScene("Game Over", font_size=48, width=400, next=1)
            director.replace(s)
        elif status == GAME_RESTART_LEVEL:
            gs = GameScene(self.level_num)
            ts = TransitScene("Restart level %d" % (self.level_num+1), font_size=48, width=600, next=gs)
            director.replace(ts)
        else:
            pass

    def show_msg(self, text):
        if self.msgs == 0:
            pos = (WIDTH-224, HEIGHT+132)
            mov = (0, -200)
        elif self.msgs == 1:
            pos = (400, HEIGHT+132)
            mov = (0, -200)
        elif self.msgs == 2:
            pos = (1300, 475)
            mov = (-500, 0)
        else:
            pos = (1300, 250)
            mov = (-500, 0)
        self.msgs += 1
        self.msgs = self.msgs % 4
        msg_box = Sprite('msg_box.png', position=pos, opacity=50)
        self.add(msg_box, z=10000)
        msg_lbl = Label(text, (-170, 40), bold=True, multiline=True, width=350, font_size=16, font_name=FONTS)
        msg_box.add(msg_lbl)

        m = MoveBy(mov, 2)
        a = Accelerate(m, 0.6)
        a = a + Delay(4) + Reverse(a) + CallFunc(self._remove_msg, msg_box)
        msg_box.do(a)

    def _remove_msg(self, msg):
        msg.parent.remove(msg)

    def _flashing(self, dt):
        f_opacity = [30, 75, 170, 255,
                     30, 75, 170, 255]#,
                     #15, 30, 50, 75, 105, 140, 180, 255]

        if self._flashing_cnt < len(f_opacity):
            self._flash_layer.opacity = f_opacity[self._flashing_cnt]
            self._flashing_cnt += 1

        if self._flashing_cnt >= len(f_opacity):
            if self.level.travelling_time:
                self._flash_layer.opacity = 50
                self._flash_layer.color = (0, 0, 255)
            else:
                self._flash_layer.color = (255, 255, 255)
                self._flash_layer.opacity = 0
                self.flash_lbl.element.text = ""
                self.unschedule(self._flashing)
                self._is_flashing = False

    def flash(self):
        if self._is_flashing:
            return
        print "flash"
        soundex.play("energy.wav")
        self._is_flashing = True
        self._flashing_cnt = 0
        self.flash_lbl.element.text = "TIME TRAVEL"
        self.schedule_interval(self._flashing, 0.05)

class GameScene(cocos.scene.Scene):
    def __init__(self, level):
        super(GameScene, self).__init__()
        self.level_num = level
        self.add(BackgroundLayer(), z=0)
        self.layer = CtrlLayer(self, level)
        self.add(self.layer, z=1)

    def on_enter(self):
        super(GameScene, self).on_enter()
        #soundex.set_music('LittleMan.mp3')
        #soundex.play_music()
        log("entering game scene: level %d" % self.level_num)

    def on_exit(self):
        super(GameScene,self).on_exit()
        #soundex.stop_music()

class TransitScene(cocos.scene.Scene):
    def __init__(self, text, font_size=16, width=800, next=None):
        super(TransitScene, self).__init__()
        layer = cocos.layer.Layer()
        txt_lbl = Label(text, (WIDTH/2, HEIGHT/2), bold=True, multiline=True, width=width, font_size=font_size, font_name=FONTS, 
                        anchor_x='center', anchor_y='center', halign='right')
        layer.add(txt_lbl)
        self.add(layer, z=0)
        if next != None:
            self.next_scene = next
            self.schedule_interval(self._transit_next_scene, 1) #TODO change to 4

    def _transit_next_scene(self, arg):
        self.unschedule(self._transit_next_scene)
        if isinstance(self.next_scene, cocos.scene.Scene):
            director.replace(self.next_scene)
        else:
            director.pop()
