import sys
import os.path

from cocos.director import director
from cocos.layer import Layer, ColorLayer, MultiplexLayer
from cocos.scene import Scene
from cocos.scenes.transitions import *
from cocos.sprite import Sprite
from cocos.menu import Menu, MenuItem, ToggleMenuItem, MultipleMenuItem, shake, shake_back, zoom_in, zoom_out
from cocos.text import Label
from cocos.actions.interval_actions import *
from cocos.actions.interval_actions import Reverse, Repeat

import pyglet
from pyglet import gl, font
from pyglet.window import key

import game_scene
import level_engine

import soundex
from constants import *
#import hiscore

class TitleLayer( Layer ):
    def __init__(self):
        super( TitleLayer, self ).__init__()

        w,h = director.get_window_size()
        self.font_title = {}

        # you can override the font that will be used for the title and the items
        self.font_title['font_name'] =  FONTS
        self.font_title['font_size'] = 26
        self.font_title['color'] = (255,204,204,255)
        self.font_title['anchor_y'] ='top'
        self.font_title['anchor_x'] ='right'

        self.font_title['color'] = (64,32,32,255)
        
        front = Sprite("menuback.png", position=(WIDTH/2, HEIGHT/2))
        # log(5) / log(2) = 2.32193
        wave_move = Accelerate(MoveBy((10,0),2),2.33)
        wave_move += MoveBy((40,0),5) 
        wave_move += MoveBy((10,0),2)
        sequence = wave_move + Delay(1) + Reverse(wave_move) + Delay(1)
        front.do(Repeat(sequence))
        self.add(front, z = 1)
        
        #front = Sprite("menufront1a.png", position=(WIDTH/2, HEIGHT/2))
        #blink = Sprite("menufront1b.png", position=(WIDTH/2, HEIGHT/2))
        #blink.do(Repeat(Blink(1,2)))
        images = (pyglet.image.load(os.path.join("data", "menufront1%s.png" % c)) for c in "ab")
        blinking = pyglet.image.Animation.from_image_sequence(images, 1)
        sprite = Sprite(blinking, position=(WIDTH/2, HEIGHT/2))

        #self.add(front, z = 2)
        self.add(sprite, z = 2)
        #bg = Sprite("chickencoop.png", position=(WIDTH/2, HEIGHT/2))
        #self.add(bg)

class OptionsMenu( Menu ):
    def __init__(self):
        super( OptionsMenu, self).__init__('Professor When')
        self.select_sound = soundex.load('move.mp3')

        # you can override the font that will be used for the title and the items
        self.font_title['font_name'] = FONTS
        self.font_title['font_size'] = 72
        self.font_title['color'] = FONT_COLOR


        self.font_item['font_name'] = FONTS
        self.font_item['color'] = FONT_COLOR
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = FONTS
        self.font_item_selected['color'] = FONT_COLOR
        self.font_item_selected['font_size'] = 46

        # example: menus can be vertical aligned and horizontal aligned
        self.menu_anchor_y = font.Text.CENTER
        self.menu_anchor_x = font.Text.CENTER

        items = []

        self.volumes = ['Mute','10','20','30','40','50','60','70','80','90','100']

        items.append( MultipleMenuItem(
                        'SFX volume: ', 
                        self.on_sfx_volume,
                        self.volumes,
                        int(soundex.sound_vol * 10) )
                    )
        items.append( MultipleMenuItem(
                        'Music volume: ', 
                        self.on_music_volume,
                        self.volumes,
                        int(soundex.music_player.volume * 10) )
                    )
        items.append( ToggleMenuItem('Show FPS:', self.on_show_fps, director.show_FPS) )
        #items.append( MenuItem('Fullscreen', self.on_fullscreen) )
        items.append( MenuItem('Back', self.on_quit) )
        self.create_menu( items, zoom_in(), zoom_out())

    def on_fullscreen( self ):
        director.window.set_fullscreen( not director.window.fullscreen )

    def on_quit( self ):
        self.parent.switch_to( 0 )

    def on_show_fps( self, value ):
        director.show_FPS = value

    def on_sfx_volume( self, idx ):
        vol = idx / 10.0
        soundex.sound_volume( vol )

    def on_music_volume( self, idx ):
        vol = idx / 10.0
        soundex.music_volume( vol )

class HelpLayer(Layer):
    FONT_SIZE=22
    is_event_handler = True #: enable pyglet's events
    def __init__(self):
        super(HelpLayer, self).__init__()
        
        # background
        self.add(Sprite("space2b.png", position=(WIDTH/2, HEIGHT/2),opacity=140,),z=-1)

        lbl_args = dict(font_name=FONTS, font_size=46, anchor_y='top', anchor_x='left', color=FONT_COLOR)
        l = Label("Help", position=(200, HEIGHT-30), **lbl_args)
        self.add(l)
        ts = []
        # please feel free to correct any inaccuracies
        ts.append("Professor When must reach      TARDIS as quick as possible")
        ts.append("You can push      crates.       boulders fall if touched.")
        ts.append("Entering a        timewarp sends you back to the beginning")
        ts.append("of a level as a new     incarnation! Now you must watch out")
        ts.append("not to get in contact with your previous incarnation or")
        ts.append("you're die:( Good luck on your way home Professor!")
        ts.append("")
        ts.append("")

        ts.append("press esc to go back")
        lbl_args = dict(font_name=FONTS, font_size=28, anchor_y='top', anchor_x='left', color=FONT_COLOR)
        for i, t in enumerate(ts):
            l = Label(t, position=(20, HEIGHT-140-i*60), **lbl_args)
            self.add(l)
        d = 180
        self.add(Sprite("booth1d.png", position=(500, HEIGHT+20-d), scale=1.5))
        self.add(Sprite("crate1a.png", position=(265, HEIGHT-40-d), scale=1.0))
        self.add(Sprite("boulder1a.png", position=(470, HEIGHT-40-d), scale=1.5))
        self.add(Sprite("energy1a1.png", position=(240, HEIGHT-100-d), scale=1.5))
        self.add(Sprite("prof1a.png", position=(380, HEIGHT-160-d), scale=1.5))

        # anchor y axis is inverted!
        galactic = Sprite("space2e.png", position=(750, HEIGHT-400-d), anchor=(100,67))  
        # how to make an infinite action?
        rotation = RotateBy( -20, duration=1)
        infinite_rotation = Repeat(rotation)
        galactic.do( infinite_rotation )
        self.add(galactic)


class ScoreLayer(Layer):
    FONT_SIZE=22
    is_event_handler = True #: enable pyglet's events
    def __init__(self):
        super(ScoreLayer, self).__init__()
        lbl_args = dict(font_name=FONTS, font_size=46, anchor_y='top', anchor_x='left', color=(255,255,0,255))
        l = Label("Scores", position=(100, HEIGHT-10), **lbl_args)
        self.add(l)
        lbl_args = dict(font_name=FONTS, font_size=22, anchor_y='top', anchor_x='left', color=(255,255,0,255))
        i = 0
        for name, score in hiscore.load():
            l = Label("%15s" % name, position=(20, HEIGHT-140-i*50), **lbl_args)
            self.add(l)
            l = Label("%d" % score, position=(620, HEIGHT-140-i*50), **lbl_args)
            self.add(l)
            i += 1
        l = Label("press esc to go back", position=(20, HEIGHT-140-11*50), **lbl_args)
        self.add(l)

class MainMenu( Menu ):

    def __init__(self):
        super( MainMenu, self).__init__('Professor When')

        self.select_sound = soundex.load('move.mp3')
        #self.activate_sound = soundex.load('LittleMan.mp3')
        soundex.set_music('theme.mp3')

        # you can override the font that will be used for the title and the items
        # you can also override the font size and the colors. see menu.py for
        # more info
        self.font_title['font_name'] = FONTS
        self.font_title['font_size'] = 72
        self.font_title['color'] = FONT_COLOR

        self.font_item['font_name'] = FONTS
        self.font_item['color'] = FONT_COLOR
        self.font_item['font_size'] = 32
        self.font_item_selected['font_name'] = FONTS
        self.font_item_selected['color'] = FONT_COLOR
        self.font_item_selected['font_size'] = 46


        # example: menus can be vertical aligned and horizontal aligned
        self.menu_anchor_y = font.Text.CENTER
        self.menu_anchor_x = font.Text.CENTER

        self.level = 0

        items = []

        items.append( MenuItem('New Game', self.on_new_game) )
        items.append( MultipleMenuItem(
                        'Level: ', 
                        self.on_level_change,
                        [ str(l+1) for l in range(len(level_engine.levels)) ],
                        0 )
                    )
        items.append( MenuItem('Options', self.on_options) )
        #items.append( MenuItem('Scores', self.on_scores) )
        items.append( MenuItem('Help', self.on_help) )
        items.append( MenuItem('Quit', self.on_quit) )

        self.create_menu(items, zoom_in(), zoom_out())

        #lbl_args = dict(font_name=FONTS, font_size=42, anchor_y='top', anchor_x='left', color=FONT_COLOR)
        #l = Label("by Fiercing Vowels", position=(300, HEIGHT-100), **lbl_args)
        #self.add(l)


    def on_new_game(self):
        gs = game_scene.GameScene(self.level)
        ts = game_scene.TransitScene("Level %d" % (self.level+1), font_size=48, width=500, next=gs)
        director.push(ts)

    def on_level_change(self, idx):
        self.level = idx

    def on_options( self ):
        self.parent.switch_to(1)

    def on_scores( self ):
        s = Scene()
        s.add(ScoreLayer())
        director.push(s)

    def on_help( self ):
        s = Scene()
        s.add(HelpLayer())
        director.push(s)

    def on_quit(self):
        pyglet.app.exit()

def get_menu_scene():
    scene = Scene()
    scene.add(MultiplexLayer(MainMenu(), OptionsMenu()), z=2) 
    scene.add(TitleLayer(), z=1)
    return scene
